"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.treeRead = exports.treeWrite = exports.subscribe = exports.readRaw = exports.writeRaw = exports.getTransactions = exports.sendPayment = exports.getLatestSequence = void 0;
const protocol_constants_1 = require("../util/protocol.constants");
const ripple_lib_1 = require("ripple-lib");
const zlib = __importStar(require("zlib"));
const util = __importStar(require("util"));
const chunkString = (str, length) => str.match(new RegExp('.{1,' + length + '}', 'gs'));
const PAYLOAD_SIZE = 925;
const debug = false;
const cloneApi = async (api) => {
    try {
        const subApi = new ripple_lib_1.RippleAPI({ server: api.connection['_url'] });
        await subApi.connect();
        return subApi;
    }
    catch (e) {
        if (debug) {
            console.log("CLONEAPI ERR", e);
        }
        return await cloneApi(api);
    }
};
const getLatestSequence = async (api, accountAddress) => {
    if (debug)
        console.log("Getting acc info for", accountAddress);
    const accountInfo = await api.getAccountInfo(accountAddress, {});
    return Number(accountInfo.sequence - 1);
};
exports.getLatestSequence = getLatestSequence;
const compressB64 = async (data) => (await util.promisify(zlib.deflate)(Buffer.from(data, 'utf-8'))).toString('base64');
const decompressB64 = async (data) => (await util.promisify(zlib.inflate)(Buffer.from(data, 'base64'))).toString('utf-8');
const sendReliably = (api, signed, preparedPayment) => new Promise((res, rej) => {
    const ledgerClosedCallback = async (event) => {
        let status;
        try {
            status = await api.getTransaction(signed.id, {
                minLedgerVersion: 25235454
            });
        }
        catch (e) {
            // Typical error when the tx hasn't been validated yet:
            if (e.name !== 'MissingLedgerHistoryError') {
                //console.log(e)
            }
            if (event.ledger_index > preparedPayment.instructions.maxLedgerVersion + 3) {
                // Assumptions:
                // - We are still connected to the same rippled server
                // - No ledger gaps occurred
                // - All ledgers between the time we submitted the tx and now have been checked for the tx
                status = {
                    finalResult: 'Transaction was not, and never will be, included in a validated ledger'
                };
                return rej(status);
            }
            else {
                // Check again later:
                api.connection.once('ledgerClosed', ledgerClosedCallback);
                return;
            }
        }
        return res(status);
    };
    api.connection.once('ledgerClosed', ledgerClosedCallback);
});
const sendPayment = async (api, data, from, to, secret, sequence) => {
    if (debug)
        console.log("Sending payment with seq", sequence);
    const options = {
        maxLedgerVersionOffset: 5,
        fee: protocol_constants_1.MIN_XRP_FEE,
        sequence: sequence,
    };
    const payment = {
        source: {
            address: from,
            maxAmount: {
                value: protocol_constants_1.MIN_XRP_TX_VALUE,
                currency: 'XRP'
            },
        },
        destination: {
            address: to,
            amount: {
                value: protocol_constants_1.MIN_XRP_TX_VALUE,
                currency: 'XRP'
            },
        },
        memos: data,
    };
    const _api = await cloneApi(api);
    try {
        const prepared = await _api.preparePayment(from, payment, options);
        const signed = _api.sign(prepared.txJSON, secret);
        const txHash = await _api.submit(signed.signedTransaction);
        //if(debug) console.log("Transaction submitted", txHash)
        await sendReliably(_api, signed, prepared);
        return txHash;
    }
    catch (error) {
        if (debug)
            console.log("SENDPAYMENT ERROR", error);
        throw error;
    }
    finally {
        _api.disconnect();
    }
};
exports.sendPayment = sendPayment;
const getTransactions = async (api, address, minLedgerVersion = 25235454) => {
    const txs = await api.getTransactions(address, {
        minLedgerVersion: minLedgerVersion,
        earliestFirst: true,
        excludeFailures: true,
    });
    return txs;
};
exports.getTransactions = getTransactions;
const writeRaw = async (api, data, from, to, secret, sequence) => {
    //if (memoSize(data) > 1000) throw new Error("data length exceeds capacity")
    try {
        if (!sequence) {
            const accountInfo = await (0, exports.getLatestSequence)(api, from);
            sequence = accountInfo + 1;
        }
        const resp = await (0, exports.sendPayment)(api, [data], from, to, secret, sequence);
        return resp['tx_json'].hash;
    }
    catch (error) {
        if (debug) {
            console.log("WRITERAW ERR", error);
        }
        throw error;
    }
};
exports.writeRaw = writeRaw;
const readRaw = async (api, hash) => {
    api = await cloneApi(api);
    let tx;
    try {
        tx = await api.getTransaction(hash, {
            minLedgerVersion: 25235454
        });
    }
    catch (e) {
        //    if(debug){
        console.log("READRAW ERR", e);
        api.isConnected;
        //    }
        throw e;
    }
    finally {
        await api.disconnect();
    }
    if (!tx || !tx.specification || !tx.specification['memos'] || !tx.specification['memos'][0]) {
        console.log(tx);
        throw new Error('Invalid Transaction ' + hash);
    }
    return tx.specification['memos'][0];
};
exports.readRaw = readRaw;
const subscribe = async (api, address, callback) => {
    api.connection.on('transaction', (tx) => callback(tx));
    await api.connection.request({
        command: 'subscribe',
        accounts: [address],
    });
};
exports.subscribe = subscribe;
const treeWrite = async (api, data, wallet, to, format = 'L') => {
    data = await compressB64(data);
    const chunks = chunkString(data, PAYLOAD_SIZE);
    const latestSequence = await (0, exports.getLatestSequence)(api, wallet.address);
    const hashes = await Promise.all(Object.entries(chunks).map(([i, chunk]) => (0, exports.writeRaw)(api, { data: chunk, format: format }, wallet.address, to, wallet.secret, latestSequence + Number(i) + 1)));
    if (hashes.length === 1) {
        return hashes[0];
    }
    return await (0, exports.treeWrite)(api, JSON.stringify(hashes), wallet, to, 'N');
};
exports.treeWrite = treeWrite;
const treeRead = async (api, hashes) => {
    const memos = await Promise.all(hashes.map(hash => (0, exports.readRaw)(api, hash)));
    const payload = await decompressB64(memos.map(memo => memo.data).join(''));
    if (memos.some(memo => memo.format === 'N')) {
        return await (0, exports.treeRead)(api, JSON.parse(payload));
    }
    return payload;
};
exports.treeRead = treeRead;
