import { xrpIO } from "xrpio";

type Wallet = {
    address: string,
    secret: string
}

type DatawriterConfig = {
    receiveAddress: string
    sendWallet: Wallet
    xrpNode: string
    contractAddress: string
}

export class Datawriter {

    constructor(private config: DatawriterConfig) {
    }


    async callEndpoint(endpointName: string, parameter: any, fee?: number) {
        const xrpio: xrpIO = new xrpIO(this.config.xrpNode)
        await xrpio.connect()
        try{
            const dataHash = await xrpio.treeWrite(
                JSON.stringify(parameter),
                this.config.receiveAddress,
                this.config.sendWallet.secret
            )
            await xrpio.writeRaw(
                {
                    data: JSON.stringify({
                        endpoint: endpointName,
                        data: dataHash
                    })
                },
                this.config.contractAddress,
                this.config.sendWallet.secret,
                undefined,
                fee ? String(fee) : undefined
            )
        }catch(e){
            console.log(e)
        }finally{
            await xrpio.disconnect()
        }
    }
}

(async () => {
    const owner_dw = new Datawriter({
        receiveAddress: "rDuXvMYNCEJCYDMFQykcafXhgi2NvMWqR",
        sendWallet: {
            address: "rUsPfG1hn6w6is28p5FUDWdMwvCo1iHrYq",
            secret: "sEdTMicaTVmfsVMLhxrufyAzEQSnsaP"
        },
        xrpNode: "wss://s.altnet.rippletest.net:51233",
        contractAddress: 'rLaXUiYvW1EMns69PsAfwSLb2VgNtPpZwq'
    })

    //await owner_dw.callEndpoint('submit', { title: "1", body: "2", from: "3" }, 100)

    const user_dw = new Datawriter({
        receiveAddress: "rwqCiEr3SLF43rAduhCChkR2K1XDhiqx5g",
        sendWallet: {
            address: "rHWN4X3hbodryX8H1EoPvmMV7AFHngEiBe",
            secret: "sEdTdLPWvz69UAUmt1zYijTyWheER9u"
        },
        xrpNode: "wss://s.altnet.rippletest.net:51233",
        contractAddress: 'rLaXUiYvW1EMns69PsAfwSLb2VgNtPpZwq'
    })

    //await owner_dw.callEndpoint('submit', { title: "1", body: "2", from: "3" }, 100)
    //await user_dw.callEndpoint('restricted', { title: "user", body: "user", from: "user" })
    //await owner_dw.callEndpoint('restricted', { title: "owner", body: "owner", from: "owner" })

    await user_dw.callEndpoint('setTns', { hash: '01708abcF00636CE10E191FD782DBDC8F4076F28404BD88EBC31DE42DD084C0944B', name: 'test'})

})()