import { z } from "zod";
import { RJSVM_Implementations, RJSVM_Interface, RJSVM, RJSVM_Config, RJSVM_InitState } from "./RJSVM/framework/types"
import { RJSVM_Builder } from "./main";
import { xrp_transaction_hash_schema } from "./util/schemas";

const shoutSchema = z.object({
    title: z.string(),
    body: z.string(),
    from: z.string(),
    id: z.optional(z.string())
})
type Shout = z.infer<typeof shoutSchema>

const tnsEntrySchema = z.object({
    hash: xrp_transaction_hash_schema,
    name: z.string(),
})
type TnsEntry = z.infer<typeof tnsEntrySchema>

type State = {
    shouts: Shout[]
}

type RJSVM_Endpoints = {
    submit: (data: Shout) => void
    restricted: (data: Shout) => void
    setTns: (entry: TnsEntry) => void
}

abstract class RJSVM_Base
extends RJSVM<State, RJSVM_Endpoints>
implements RJSVM_Interface<RJSVM_Base> {
    owner = "rUsPfG1hn6w6is28p5FUDWdMwvCo1iHrYq"

    state: State = {
        shouts: []
    }
}

const RJSVM_Contract: RJSVM_Implementations<RJSVM_Base> = {
    submit: {
        implementation: function (env, shout) {
            this.state.shouts.unshift(shout)
            console.log(shout)
        },
        visibility: 'public',
        fee: 10,
        parameterSchema: shoutSchema
    },

    restricted: {
        implementation: function (env, shout) {
            this.state.shouts.unshift(shout)
            console.log(shout)
        },
        visibility: 'owner',
        parameterSchema: shoutSchema
    },
    setTns: {
        implementation: function (env, tnsEntry) {
            console.log(tnsEntry)
        },
        visibility: 'public',
        parameterSchema: tnsEntrySchema,
    }
}


const Rjsvm = RJSVM_Builder.from(RJSVM_Base, RJSVM_Contract);

const conf: RJSVM_Config = {
    listeningAddress: "rLaXUiYvW1EMns69PsAfwSLb2VgNtPpZwq",
    rippleNode: "wss://s.altnet.rippletest.net:51233"
}

const rjsvm = new Rjsvm(conf)

rjsvm.connect()